//
// GameX - Debug Class Code 
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source License. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

#include <stdio.h>
#include <stdlib.h>

//*** Global Debugging (for ALL modules)
// The following define tells the debug include file that
// THIS module (Debug.cpp) should recieve the 
// global "debug" object to make debugging available to 
// all modules automatically. All other modules (except the 
// Debug module) will receive an external reference to this 
// global variable. (See debug.hpp)

#define DEBUG_GLOBAL

//*** Debugging Include

#include "gamex-debug.hpp"

#include <string.h>
#include <stdarg.h>

//*** Debugging Class Code

Debugging::Debugging (void)
{
	m_output_file = NULL;
	Open ("debug.txt");
	fprintf (m_output_file, "* Debugging Output *\n");
	m_nonEmpty = false;
}

Debugging::~Debugging (void)
{
	Close ();
	m_output_file = NULL;
}

void Debugging::Open (char *filename)
{
	if (m_output_file!=NULL) fclose (m_output_file);
	m_output_file = fopen (filename, "w+t");
	if (m_output_file==NULL) {
		printf ("DEBUGGER ERROR: Unable to open debug log file.\n");
	}
}

void Debugging::Close (void)
{
	if (m_output_file!=NULL) {
		fclose (m_output_file);
		if(!m_nonEmpty) remove("debug.txt");
	}
}

void Debugging::Output (const char* frm, ...)
{
	va_list ap; // variable arguments pointer
	va_start(ap, frm);

	vfprintf(m_output_file, frm, ap);
	fflush (m_output_file);

	va_end(ap);
	m_nonEmpty = true;
}

/*
// Creates and returns a new string based on a sprintf-style formatted string and variable parameters
// Can be adapted to allow any function (not just file-printing ones) to take formatted input
// Note that as this is, the returned string must be deallocated with delete[] afterwards to avoid memory leakage
char* newStringFromFormat (const char* frm, ...)
{
	int len  = strlen(frm) + 32;
	char* str = new char [len];

	va_list ap; // variable arguments pointer
	va_start(ap, frm);

	while(_vsnprintf(str, len, frm, ap) < 0) {
		len *= 2; // double the length until print succeeds
		delete[] str, str = new char [len];
	}

	va_end(ap);
	return str;
}
*/
